(*
 * InnerProduct.m
 *
 * Andrew D. Lewis
 * Queen's University
 * 14 May 2001
 *
 * Mathematica for manipulating things involving inner products.
 *
 *)

BeginPackage["InnerProduct`"]

Angle::usage = "Angle[x,y,g] returns the cosine of the angle between objects
x and y relative to the inner product g.  The inner product is a function
taking objects of the desired type and returning a number.  For normal
vectors, take g[x_,y_]:=x.y"

GramSchmidt::usage = "GramSchmidt[A,g] computes the orthogonalisation of the
objects in A with respect to the inner product g.  In A are objects of any
sort, as long as the inner product is a function taking two such objects and
giving a number.  For normal vectors, take g[x_,y_]:=x.y"

Normalise::usage = "Normalise[A,g] normalises the objects in A with respect
to the inner product g.  In A are objects of any sort, as long as the inner
product is a function taking two such objects and giving a number.  For
normal vectors, take g[x_,y_]:=x.y"

OrthogonalComplement::usage = "OrthogonalComplement[A,g] computes the
orthogonal complement of the objects in A with respect to the inner product
g.  In A are objects of any sort, as long as the inner product is a function
taking two such objects and giving a number.  For normal vectors, take
g[x_,y_]:=x.y.  For the results to be meaningful, the objects in A must be
orthonormal."

OrthogonalProjection::usage = "OrthogonalProjection[A,g] computes the
orthogonal projection onto the subspace of objects contained in A with
respect to the inner product g.  In A are objects of any sort, as long as the
inner product is a function taking two such objects and giving a number.  For
normal vectors, take g[x_,y_]:=x.y.  For the results to be meaningful, the
objects in A must be orthonormal."

SymmetricQ::usage = "SymmetricQ[A,g,tools] returns True if the linear
operator A is symmetric with respect to the inner product defined by g.  A
should be a a Mathematica function that takes objects to objects.  Anything
needed to define A should be in the list tools.  The objects that A takes can
be of any sort, as long as the inner product is a function taking two such
objects and giving a number.  For normal vectors, take tools={L},
A[tools,x_]:=tools[[1]].x, and g[x_,y_]:=x.y.  Here L is a regular old
matrix, so the A is simply multiplication by L."

Begin["`Private`"]

Print["\nPackage \"InnerProduct\" defines: Angle, GramSchmidt, Normalise, OrthogonalComplement, OrthogonalProjection, SymmetricQ."];

Angle[x_,y_,g_] := Module[{},
  g[x,y]/(Sqrt[g[x,x]]Sqrt[g[y,y]])
]

OrthogonalComplement[A_,g_] := Module[{},
  NullSpace[A.g]
]

GramSchmidt[A_, g_] := Module[{B,i,j,m,temp},
  B = Table[0,{i,Length[A]}];
  Do[
    temp = 0;
    temp = A[[m]] - Sum[g[B[[i]],A[[m]]] B[[i]],{i, m-1}];
    B[[m]] = temp/Sqrt[g[temp,temp]],
    {m, Length[A]}
  ];
  Table[B[[i]],{i,Length[A]}]
]

Normalise[A_,g_] := Module[{B},
  B = Table[0,{i,Length[A]}];
  Do[B[[m]] = A[[m]]/Sqrt[G[A,A]],{m,Length[A]}];
  Table[B[[i]],{i,Length[A]}]
]

OrthogonalProjection[A_,g_] := Module[{v,a,B},
  v = Table[a[i],{i,Length[g]}];
  B = Sum[A[[i]].g.v A[[i]],{i,Length[A]}];
  Table[Coefficient[Expand[B[[i]]],a[j]],{i,Length[g]},{j,Length[g]}]
]

SymmetricQ[A_,g_,tools_] := Module[{B1,B2,Zero,i,j},
(*
 * Indicates whether A is symmetric with respect to g.
 *)
   B1 = Transpose[g].A;
   B2 = Transpose[g.A];
   Zero = Table[0,{i,Length[g]},{j,Length[g]}];
   If[SameQ[Simplify[B1-B2],Zero],True,False]
]

End[]
EndPackage[]
